SystemOrganization addCategory: #'Signals-Core'!
SystemOrganization addCategory: #'Signals-Benchmarks'!
SystemOrganization addCategory: #'Signals-Tests'!
SystemOrganization addCategory: #'Signals-Wrapper'!
SystemOrganization addCategory: #'Signals-Wrapper-Tests'!

!ClassDescription methodsFor: '*signals-ob-morphic' stamp: 'mt 6/9/2010 09:02:14.076'!
browserIcon: aClassDescription selector: aSelector
	"Answer an icon to be shown for the selector aSymbol in aClassDescription."

	aClassDescription methodDictionary at: aSelector ifPresent: [ :method |
		method literalsDo: [ :literal |
			(literal == #halt or: [ literal == #halt: or: [ literal == #haltIfNil or: [ literal == #haltIf: or: [ literal == #haltOnce or: [ literal == #haltOnce: or: [ literal == #haltOnCount: or: [ literal == #halt:onCount: ] ] ] ] ] ] ])
				ifTrue: [ ^ #breakpoint ].
			(literal == #flag: or: [ literal == #needsWork or: [ literal == #notYetImplemented ] ])
				ifTrue: [ ^ #flag ].
			"BEGIN: Signal extension"
			literal == #emit
				ifTrue: [^ #signal ].
			literal == #emitAlways
				ifTrue: [^ #signal2 ]
			"END: Signal extension" ] ].
	^ (aClassDescription isOverride: aSelector)
		ifTrue: [
			(aClassDescription isOverridden: aSelector)
				ifTrue: [ #arrowUpAndDown ]
				ifFalse: [ #arrowUp ] ]
		ifFalse: [
			(aClassDescription isOverridden: aSelector)
				ifTrue: [ #arrowDown ]
				ifFalse: [ #blank ] ]! !

WeakIdentityKeyDictionary subclass: #SignalConnectionsRepository
	instanceVariableNames: 'mutex'
	classVariableNames: 'Repository'
	poolDictionaries: ''
	category: 'Signals-Core'!

!SignalConnectionsRepository commentStamp: 'mt 6/2/2010 22:43' prior: 0!
I store all active signal-selector connections with weak data structures.!

!SignalConnectionsRepository class methodsFor: 'testing' stamp: 'mt 6/14/2010 21:37:53.926'!
checkSignal: aSignal inClass: aClass
	"Check whether there are connections for the signal in the class."
	
	self critical: [
		self instance associationsDo: [:assoc |
			assoc key ifNotNil: [:sender | (sender class = aClass and: [
					assoc value keys anySatisfy: [:signalKey |
						signalKey selector = aSignal]]) ifTrue: [^ true]]]].

	^ false! !

!SignalConnectionsRepository class methodsFor: 'instance' stamp: 'mt 6/3/2010 08:50'!
critical: aBlock

	^ self instance critical: aBlock! !

!SignalConnectionsRepository class methodsFor: 'instance' stamp: 'mt 6/14/2010 20:43:52.302'!
instance

	^ Repository ifNil: [Repository := self new. WeakArray addWeakDependent: Repository. Repository]! !

!SignalConnectionsRepository class methodsFor: 'instance' stamp: 'mt 6/14/2010 20:47:18.789'!
reset

	WeakArray removeWeakDependent: Repository.
	Repository := nil.! !

!SignalConnectionsRepository methodsFor: 'critical' stamp: 'mt 6/3/2010 08:46'!
critical: aBlock

	mutex ifNil: [mutex := Mutex new].
	^ mutex critical: aBlock! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:23'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector
	"Mediator: Connect a signaling object with a receiver."

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: nil
		queue: nil
		blocking: false! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:23'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector pattern: argPattern
	"Shadow or reorder arguments before invoking selector on receiver, e.g.,  #(1 4 3)"

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: argPattern
		queue: nil
		blocking: false! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:23'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector pattern: argPattern queue: aSharedQueue
	"Shadow or reorder arguments before invoking selector on receiver, e.g.,  #(1 4 3)"

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: argPattern
		queue: aSharedQueue
		blocking: false! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:17'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector pattern: argPattern queue: aSharedQueue blocking: aBoolean
	"Shadow or reorder arguments before invoking selector on receiver, e.g.,  #(1 4 3)"

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: argPattern
		queue: aSharedQueue
		blocking: aBoolean! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:25'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector queue: aSharedQueue
	"Store an incoming signal connection request. This is a direct connection, so the receiver will be called synchonously after the signal was emitted. See #emit."
	
	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: nil
		queue: aSharedQueue
		blocking: false! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:18'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector queue: aSharedQueue blocking: aBoolean

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: nil
		queue: aSharedQueue
		blocking: aBoolean! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:23'!
connect: aSender signal: aSignal toSelector: aSelector
	"Connect a selector from me with the signal of a sender."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: nil
		queue: nil
		blocking: false.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:24'!
connect: aSender signal: aSignal toSelector: aSelector pattern: argPattern
	"Connect a selector from me with the signal of a sender."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: argPattern
		queue: nil
		blocking: false.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:24'!
connect: aSender signal: aSignal toSelector: aSelector pattern: argPattern queue: aSharedQueue
	"Create a queued connection. Receiver will not be called directly but a valuable block with the call will be stored into a queue. The queue has to be processed somewhere else. See #emit."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: argPattern
		queue: aSharedQueue
		blocking: false.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:20'!
connect: aSender signal: aSignal toSelector: aSelector pattern: argPattern queue: aSharedQueue blocking: aBoolean
	"Create a queued connection. Receiver will not be called directly but a valuable block with the call will be stored into a queue. The queue has to be processed somewhere else. See #emit."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: argPattern
		queue: aSharedQueue
		blocking: aBoolean.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:24'!
connect: aSender signal: aSignal toSelector: aSelector queue: aSharedQueue
	"Connect a selector from me with the signal of a sender."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: nil
		queue: aSharedQueue
		blocking: false.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:20'!
connect: aSender signal: aSignal toSelector: aSelector queue: aSharedQueue blocking: aBoolean
	"Connect a selector from me with the signal of a sender."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: nil
		queue: aSharedQueue
		blocking: aBoolean.! !

!Object methodsFor: '*signals-core-private' stamp: 'mt 6/14/2010 20:31:46.453'!
connectSignal: aSignal to: aReceiver selector: aSelector pattern: argPattern queue: aSharedQueue blocking: aBoolean
	"Private. "

	| forceUiProcess numArgsSender numArgsReceiver fixedPattern |
	((self respondsTo: aSignal)
		and: [aReceiver respondsTo: aSelector])
		ifFalse: [Error signal: 'Could not connect sender to receiver!!'].
	
	numArgsSender := aSignal count: [:c | c = $:].
	numArgsReceiver := aSelector count: [:c | c = $:].
	(numArgsSender < numArgsReceiver and: [argPattern isNil])
		ifTrue: [Error signal: 'Could not connect sender to receiver!! Receiver expects more arguments!!'].
	
	argPattern ifNotNil: [
		((argPattern isEmpty or: [argPattern min >= 0])
			and: [numArgsReceiver == argPattern size]
			and: [argPattern isEmpty or: [numArgsSender >= argPattern max]])
			ifFalse: [Error signal: 'Could not connect sender to receiver!! Pattern is invalid!!']].
	
	"Auto-truncate unnecessary arguments."
	(numArgsSender > numArgsReceiver and: [argPattern isNil])
		ifTrue: [fixedPattern := (1 to: numArgsReceiver) collect: [:i | i]]
		ifFalse: [fixedPattern := argPattern].

	"Signals sent in a non-multiprogramming environment should be processed in the UI process no matter from which process they are emitted."
	forceUiProcess := Processor activeProcess == Project uiProcess and: [aSharedQueue isNil].
	
	"Add the connection."
	SignalConnectionsRepository critical: [
		(self signalConnections includesKey: (SignalKey signal: aSignal sender: self))
			ifFalse: [self signalConnections
				at: (SignalKey signal: aSignal sender: self) configureFinalize
				put: WeakIdentityKeyDictionary new].
		
		((self signalConnections
			at: (SignalKey signal: aSignal sender: self))
			at: aReceiver
			ifAbsentPut: [OrderedCollection new])
			add: (SignalConnectionConfiguration new
				selector: aSelector;
				pattern: fixedPattern;
				queue: aSharedQueue;
				isBlocking: aBoolean;
				forceUiProcess: forceUiProcess)].
		
	"Signals-Wrapper extension --- If signal is not a signal, make it to a signal."
	(Smalltalk includesKey: #SignalMethodWrapper)
		ifTrue: [SignalMethodWrapper convertToSignal: aSignal inClass: self class].! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/6/2010 23:05:03.457'!
createAutoSignalConnections
	"Transform all selectors like #onSenderSignal:arg1:arg2: into corresponding connection create statements and execute them. The sender has to be an instance variable with the same name and has to be initialized yet."
	
	| senderSignalPatterns |
	senderSignalPatterns := self class selectors
		select: [:sel | sel startsWith: #on]
		thenCollect: [:sel | sel allButFirst: 2].
	
	self class allInstVarNames do: [:var | 
		senderSignalPatterns
			select: [:pattern | pattern withFirstCharacterDownshifted startsWith: var]
			thenDo: [:pattern |
				| sender signal |
				sender := self instVarNamed: var.
				signal := (pattern allButFirst: var size) withFirstCharacterDownshifted asSymbol.
				(sender respondsTo: signal) ifTrue: [ "enable 'lineEdit' and 'lineEditPassword' inst var names"
					self
						connect: sender
						signal: signal
						toSelector: (#on, pattern) asSymbol]]].! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/2/2010 22:41'!
disconnectAllSignals
	
	SignalConnectionsRepository instance
		removeKey: self
		ifAbsent: [^ false].
		
	^ true! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/14/2010 19:45:19.609'!
disconnectSignal: aSignal
	
	self signalConnections
		removeKey: (SignalKey signal: aSignal sender: self)
		ifAbsent: [^ false].
		
	^ true! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/14/2010 19:46:18.6'!
disconnectSignal: aSignal from: aReceiver
	
	(self signalConnections includesKey: (SignalKey signal: aSignal sender: self))
		ifFalse: [^ false].
		
	(self signalConnections at: (SignalKey signal: aSignal sender: self))
		removeKey: aReceiver
		ifAbsent: [^ false].
	
	(self signalConnections at: (SignalKey signal: aSignal sender: self))
		ifEmpty: [^ self disconnectSignal: aSignal].
		
	^ true! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/14/2010 19:46:38.42'!
disconnectSignal: aSignal from: aReceiver selector: aSelector
	
	(self signalConnections includesKey: (SignalKey signal: aSignal sender: self))
		ifFalse: [^ false].
		
	((self signalConnections at: (SignalKey signal: aSignal sender: self))
		at: aReceiver
		ifAbsent: [^ false]) "weak"
		removeAllSuchThat: [:connectionConfiguration | connectionConfiguration selector = aSelector].
	
	((self signalConnections at: (SignalKey signal: aSignal sender: self))
		at: aReceiver
		ifAbsent: [^ true]) "weak"
		ifEmpty: [self disconnectSignal: aSignal from: aReceiver].
	
	^ true! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/14/2010 19:58:55.274'!
emit
	"Calling this from a method makes the method to a signal."

	| signalCtx signal args |
	signalCtx := thisContext sender.
	signal := signalCtx method.
	
	"Do not allow external emitting of signals."
	signalCtx sender receiver == signalCtx receiver
		ifFalse: [Error signal: 'Must not emit an object''s signal from the outside!!'].
	
	"Extract arguments."
	args := Array new: signal numArgs.
	(1 to: signal numArgs) do: [:index |
		args at: index put: (signalCtx tempAt: index)].
	
	self emitSignal: signal selector arguments: args sender: self.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/14/2010 19:59:04.461'!
emitAlways
	"Less restrictive version of #emit, e.g., does not check if sender and receiver are the same object. Makes signals emittable from the outside."
	
	| signalCtx signal args |
	signalCtx := thisContext sender.
	signal := signalCtx method.
	
	"Extract arguments."
	args := Array new: signal numArgs.
	(1 to: signal numArgs) do: [:index |
		args at: index put: (signalCtx tempAt: index)].
	
	self emitSignal: signal selector arguments: args sender: self.! !

!Object methodsFor: '*signals-core-private' stamp: 'mt 6/14/2010 22:04:25.276'!
emitSignal: signal arguments: args sender: aSender
	"Private. Performs the triggering of callbacks."

	| actions |
	
	"Fetch targets and schedule dispatch."
	actions := OrderedCollection new. "Buffer actions to free the mutex in time."
	SignalConnectionsRepository critical: [
		(aSender signalConnections at: (SignalKey signal: signal sender: aSender) ifAbsent: [^ self]) associationsDo: [ :association |
			association key ifNotNil: [ :receiver | "Don't let the key go away -> weak dictionary"
				association value do: [ :connectionConfiguration |
					| selector pattern queue filteredArgs blocking forceUiProcess |
					selector := connectionConfiguration selector.
					pattern := connectionConfiguration pattern.
					queue := connectionConfiguration queue.
					blocking := connectionConfiguration isBlocking.
					forceUiProcess := connectionConfiguration forceUiProcess.
					
					forceUiProcess ifTrue: [
						blocking := false. "Never block here."
						queue := Processor activeProcess == Project uiProcess
							ifTrue: [nil] "synchron processing in UI process"
							ifFalse: [WorldState deferredUIMessages]]. "asynchron processing in UI process"
					
					pattern
						ifNil: [ "no argument changes"
							filteredArgs := args]
						ifNotNil: [ "shadowing and reordering"
							filteredArgs := Array new: pattern size.
							pattern withIndexDo: [:source :target | 
								source = 0 "Send myself as argument"
									ifTrue: [filteredArgs at: target put: aSender]
									ifFalse: [filteredArgs at: target put: (args at: source)]]].
					actions add: (queue "queued connection or direct connection?"
						ifNil: [[receiver perform: selector withArguments: filteredArgs]]
						ifNotNil: [
							blocking
								ifTrue: [ | thisProcess |
									thisProcess := Processor activeProcess.
									[queue nextPut: [receiver perform: selector withArguments: filteredArgs. thisProcess resume].  thisProcess suspend]]
								ifFalse: [ [queue nextPut: [receiver perform: selector withArguments: filteredArgs]] ] ] ) ] ] ] ].
	
	"Dispatch outside the mutex."
	actions do: [:action | action value].
	
	"TODO: Handle dispatching errors with automatic disconnections."
	! !

!Object methodsFor: '*signals-core-private' stamp: 'mt 6/2/2010 22:41'!
signalConnections

	^ SignalConnectionsRepository instance
		at: self
		ifAbsentPut: [Dictionary new]! !

Object subclass: #SignalBenchmark
	instanceVariableNames: 'sender receivers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmark class methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:32:50.338'!
run

	^ self new run! !

!SignalBenchmark methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:53:28.793'!
needsPreparation

	^ true! !

!SignalBenchmark methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:41:40.031'!
prepareOneBindingFor: aReceiver

	self subclassResponsibility.! !

!SignalBenchmark methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 18:18:41.446'!
receiverCount

	^ 10000! !

!SignalBenchmark methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:57:22.712'!
run

	| preparationTime triggeringTime |
	self setUp.
	
	self receiverCount timesRepeat: [
		(receivers add: SignalBenchmarkReceiver new)].
	
	preparationTime := self needsPreparation
		ifTrue: [ [receivers do: [:receiver | self prepareOneBindingFor: receiver]] timeToRunWithoutGC ]
		ifFalse: [0]..
	
	triggeringTime := [self triggerOneEventWith: 42] timeToRunWithoutGC.
	
	"Verify validity of results."
	receivers do: [:receiver | receiver checkValue: 42].
	
	self tearDown.
	^ self class -> {preparationTime. triggeringTime}! !

!SignalBenchmark methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:41:03.411'!
setUp

	sender := SignalBenchmarkSender new.
	receivers := OrderedCollection new.! !

!SignalBenchmark methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:10:43.249'!
tearDown

	sender := nil.
	receivers := nil.! !

!SignalBenchmark methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:44:39.496'!
triggerOneEventWith: aValue

	self subclassResponsibility.! !

SignalBenchmark subclass: #SignalBenchmarkAnnouncements
	instanceVariableNames: 'announcer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkAnnouncements methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 18:02:59.378'!
initialize

	super initialize.
	announcer := AXAnnouncer new.! !

!SignalBenchmarkAnnouncements methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 18:15:46.584'!
prepareOneBindingFor: aReceiver

	announcer
		when: SignalBenchmarkAnnouncement
		do: [:announcement | aReceiver update: announcement value].
		! !

!SignalBenchmarkAnnouncements methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 18:12:50.9'!
triggerOneEventWith: aValue

	announcer announce: (SignalBenchmarkAnnouncement value: aValue).! !

SignalBenchmark subclass: #SignalBenchmarkManual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkManual methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:53:36.678'!
needsPreparation

	^ false! !

!SignalBenchmarkManual methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:50:19.755'!
prepareOneBindingFor: anObject

	"No bindings here."! !

!SignalBenchmarkManual methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:50:45.707'!
triggerOneEventWith: aValue

	receivers do: [:receiver | receiver update: aValue].! !

SignalBenchmark subclass: #SignalBenchmarkObjectDependents
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkObjectDependents methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:42:00.976'!
prepareOneBindingFor: aReceiver

	sender addDependent: aReceiver.! !

!SignalBenchmarkObjectDependents methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:44:50.082'!
triggerOneEventWith: aValue

	sender changed: aValue. "Calls #update: in receivers."! !

SignalBenchmark subclass: #SignalBenchmarkObjectEvents
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkObjectEvents methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:58:41.653'!
prepareOneBindingFor: aReceiver

	sender
		when: #valueChanged
		send: #update:
		to: aReceiver.! !

!SignalBenchmarkObjectEvents methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:59:23.643'!
triggerOneEventWith: aValue

	sender triggerEvent: #valueChanged with: aValue.! !

SignalBenchmark subclass: #SignalBenchmarkSignals
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkSignals methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:42:21.806'!
prepareOneBindingFor: aReceiver

	aReceiver
		connect: sender
		signal: #valueChanged:
		toSelector: #update:! !

!SignalBenchmarkSignals methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:45:00.615'!
triggerOneEventWith: aValue

	sender value: aValue. "Triggers #valueChanged:."! !

SignalBenchmarkSignals subclass: #SignalBenchmarkSignalsWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkSignalsWrapper methodsFor: 'as yet unclassified' stamp: 'mt 6/14/2010 22:30:39.73'!
prepareOneBindingFor: aReceiver

	aReceiver
		connect: sender
		signal: #value2:
		toSelector: #update:! !

!SignalBenchmarkSignalsWrapper methodsFor: 'as yet unclassified' stamp: 'mt 6/14/2010 22:40:32.945'!
triggerOneEventWith: aValue

	sender value2: aValue.! !

Object subclass: #SignalBenchmarkReceiver
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkReceiver methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:40:39.767'!
checkValue: expected

	value ~= expected ifTrue: [Error signal: 'Value check failed!!'].! !

!SignalBenchmarkReceiver methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 18:16:11.579'!
update: aValue

	value := aValue.! !

Object subclass: #SignalBenchmarkSender
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkSender methodsFor: 'accessing' stamp: 'mt 6/14/2010 22:30:09.622'!
value2: aValue
	"Use wrapped signals here."
! !

!SignalBenchmarkSender methodsFor: 'accessing' stamp: 'mt 6/10/2010 17:39:08.142'!
value: aValue

	self valueChanged: aValue.! !

!SignalBenchmarkSender methodsFor: 'signals' stamp: 'mt 6/10/2010 17:39:12.459'!
valueChanged: aValue

	 self emit.! !

Object subclass: #SignalBenchmarkSuite
	instanceVariableNames: 'results'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkSuite class methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:36:07.906'!
run

	^ self new run! !

!SignalBenchmarkSuite methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:36:35.728'!
initialize

	results := OrderedCollection new.! !

!SignalBenchmarkSuite methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 18:18:31.135'!
results

	^ results! !

!SignalBenchmarkSuite methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:37:16.754'!
run

	SignalBenchmark subclassesDo: [:benchmark |
		self run: benchmark].
	
	^ results! !

!SignalBenchmarkSuite methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 17:37:01.093'!
run: aBenchmark

	^ results add: aBenchmark run! !

Object subclass: #SignalConnectionConfiguration
	instanceVariableNames: 'selector pattern queue isBlocking forceUiProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Core'!

!SignalConnectionConfiguration commentStamp: 'mt 6/14/2010 19:16:09.16' prior: 0!
A data class that stores connection configuration.!

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:06:43.409'!
forceUiProcess
	"Process callback blocking if already in UI process, enqueue in WorldState>>deferredUIMessages in the other case."

	^ forceUiProcess! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:06:52.015'!
forceUiProcess: aBoolean

	forceUiProcess := aBoolean.! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:12:50.2'!
isBlocking
	"Does a queued connection wait until the responsible process performed the callback?"

	^ isBlocking! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:13:39.501'!
isBlocking: aBoolean

	isBlocking := aBoolean.! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:14:14.619'!
pattern
	"Choose and reorder arguments, e.g., #(2 1 3) or access sender with: #(0)."

	^ pattern! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:14:28.736'!
pattern: anArrayWithIntegers

	pattern := anArrayWithIntegers.! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:14:46.089'!
queue
	"A shared queue for process synchronization."

	^ queue! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:14:54.387'!
queue: aSharedQueue

	queue := aSharedQueue.! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:15:20.266'!
selector

	^ selector! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:15:43.137'!
selector: aSymbol

	selector := aSymbol.! !

Object subclass: #SignalConsumer
	instanceVariableNames: 'value switch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalConsumer methodsFor: 'slots' stamp: 'mt 6/2/2010 21:43'!
switchState

	^ switch ifNil: [switch := #off]! !

!SignalConsumer methodsFor: 'slots' stamp: 'mt 6/2/2010 21:44'!
toggleSwitch

	switch := self switchState = #off
		ifTrue: [#on]
		ifFalse: [#off].! !

!SignalConsumer methodsFor: 'slots' stamp: 'mt 6/2/2010 21:43'!
value

	^ value! !

!SignalConsumer methodsFor: 'slots' stamp: 'mt 6/6/2010 09:01:01.648'!
value: aValue

	value := aValue.! !

!SignalConsumer methodsFor: 'signals' stamp: 'mt 6/13/2010 10:40:27.55'!
valueConsumed

	self emitAlways. "public signal"! !

SignalConsumer subclass: #SignalConsumerAuto
	instanceVariableNames: 'producer producerTwo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalConsumerAuto commentStamp: 'mt 6/6/2010 22:30:38.308' prior: 0!
I am a signal consumer that makes use of auto-connected signals.!

!SignalConsumerAuto methodsFor: 'initialize-release' stamp: 'mt 6/6/2010 23:06:23.56'!
initialize

	super initialize.
	producer := SignalProducer new.
	producerTwo := SignalProducer new.
	
	self createAutoSignalConnections.! !

!SignalConsumerAuto methodsFor: 'auto-connected actions' stamp: 'mt 6/6/2010 22:24:12.93'!
onProducerNewSignalProduced
	"sender: producer signal: #newSignalProduced"
	
	self toggleSwitch.! !

!SignalConsumerAuto methodsFor: 'auto-connected actions' stamp: 'mt 6/6/2010 23:06:13.035'!
onProducerTwoNewSignalProduced
	
	self toggleSwitch.! !

!SignalConsumerAuto methodsFor: 'auto-connected actions' stamp: 'mt 6/6/2010 22:25:06.762'!
onProducerValueChanged: aValue
	"sender: producer signal: #valueChanged:"
	
	self value: aValue.! !

!SignalConsumerAuto methodsFor: 'auto-connected actions' stamp: 'mt 6/6/2010 22:25:25.597'!
onProducerValueChanged: aValue and: anotherValue
	"sender: producer signal: #valueChanged:and:"
	
	self value: anotherValue.! !

!SignalConsumerAuto methodsFor: 'accessing' stamp: 'mt 6/6/2010 22:30:18.165'!
producer

	^ producer! !

!SignalConsumerAuto methodsFor: 'accessing' stamp: 'mt 6/6/2010 23:05:53.97'!
producerTwo

	^ producerTwo! !

Object subclass: #SignalKey
	instanceVariableNames: 'class selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Core'!

!SignalKey commentStamp: 'mt 6/14/2010 19:37:06.469' prior: 0!
I am a key used in the dictionary that holds all connections for a sending object in the connections repository.!

!SignalKey class methodsFor: 'instance creation' stamp: 'mt 6/14/2010 19:38:37.422'!
signal: aSignal sender: aSender

	^ self new
		theClass: aSender class;
		selector: aSignal.! !

!SignalKey methodsFor: 'comparing' stamp: 'mt 6/14/2010 19:49:03.511'!
= anotherKey

	^ anotherKey class = self class
		and: [{self theClass. self selector} = {anotherKey theClass. anotherKey selector}]! !

!SignalKey methodsFor: 'finalization' stamp: 'mt 6/14/2010 21:40:57.545'!
configureFinalize
	"What should happen, if I die?"

	(Smalltalk includesKey: #SignalMethodWrapper)
		ifTrue: [self toFinalizeSend: #checkAndConvertSignal: to: SignalMethodWrapper with: {self selector. self theClass}].! !

!SignalKey methodsFor: 'comparing' stamp: 'mt 6/14/2010 19:42:22.85'!
hash

	^ {self theClass. self selector} hash! !

!SignalKey methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:40:11.679'!
selector

	^ selector! !

!SignalKey methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:39:53.506'!
selector: aSelector

	selector := aSelector.! !

!SignalKey methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:39:35.009'!
theClass

	^ class! !

!SignalKey methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:39:42.439'!
theClass: aClass

	class := aClass.! !

Object subclass: #SignalProducer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalProducer methodsFor: 'signals' stamp: 'mt 6/2/2010 21:39'!
newSignalProduced

	self emit.! !

!SignalProducer methodsFor: 'signals' stamp: 'mt 6/6/2010 12:20:50.329'!
newSignalProducedInBlock
	"Bad style."
	
	true
		ifTrue: [self emit]
		ifFalse: ["Do not emit."].! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 12:19:30.403'!
produceBadSignal

	self newSignalProducedInBlock.! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 09:05:10.543'!
produceSignal

	self newSignalProduced.! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 09:04:58.828'!
value: aValue

	self valueChanged: aValue.! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 09:05:26.631'!
value: aValue and: anotherValue

	self valueChanged: aValue and: anotherValue.! !

!SignalProducer methodsFor: 'signals' stamp: 'mt 6/9/2010 08:47:27.027'!
valueChanged2: aValue
	"Public emittable signal."
	
	self emitAlways.! !

!SignalProducer methodsFor: 'signals' stamp: 'mt 6/2/2010 22:20'!
valueChanged: aValue and: anotherValue

	self emit.! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 09:34:05.91'!
valueInBlock: aValue

	[self valueChanged: aValue] value.! !

SignalProducer subclass: #SignalProducerEx
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalProducerEx commentStamp: 'mt 6/17/2010 15:12:46.235' prior: 0!
Used to test base class signal emitting.!

!SignalProducerEx methodsFor: 'actions' stamp: 'mt 6/17/2010 15:14:56.043'!
valueEx: newValue
	"Emit base class signal."

	self valueChanged: newValue.! !

Object subclass: #SignalProducerByWrapping
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Wrapper-Tests'!

!SignalProducerByWrapping commentStamp: 'mt 6/10/2010 22:53:13.035' prior: 0!
I am a normal value holder.!

!SignalProducerByWrapping class methodsFor: 'wrappers' stamp: 'mt 6/10/2010 22:56:16.402'!
wrapperCount

	^ SignalMethodWrapperRepository instance
		count: [:wrapper | wrapper definingClass == self]! !

!SignalProducerByWrapping methodsFor: 'accessing' stamp: 'mt 6/10/2010 22:53:24.965'!
value

	^ value! !

!SignalProducerByWrapping methodsFor: 'accessing' stamp: 'mt 6/14/2010 18:03:46.994'!
value: aValue
	"Should be used as a signal in tests."

	value := aValue.! !

Object subclass: #SignalSpy
	instanceVariableNames: 'queue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Core'!

!SignalSpy commentStamp: 'mt 6/3/2010 11:55' prior: 0!
Useful for testing signals.!

!SignalSpy class methodsFor: 'instance creation' stamp: 'mt 6/3/2010 12:24'!
onSender: aSender signal: aSignal

	^ self new
		addSender: aSender
		signal: aSignal! !

!SignalSpy methodsFor: 'spying' stamp: 'mt 6/4/2010 13:00'!
addSender: aSender signal: aSignal

	[self
		connect: aSender
		signal: aSignal
		toSelector: aSignal] forkAndWait. "Avoid processing of signals in UI process."! !

!SignalSpy methodsFor: 'spying' stamp: 'mt 6/3/2010 12:02'!
doesNotUnderstand: aMessage

	self queue nextPut: {aMessage selector. aMessage arguments}.! !

!SignalSpy methodsFor: 'spying' stamp: 'mt 6/3/2010 11:58'!
queue

	^ queue ifNil: [queue := SharedQueue new]! !

!SignalSpy methodsFor: 'spying' stamp: 'mt 6/3/2010 12:00'!
respondsTo: aSelector

	^ true! !

!SignalSpy methodsFor: 'waiting' stamp: 'mt 6/3/2010 12:07'!
waitForNextSignal

	^ self queue next! !

!SignalSpy methodsFor: 'waiting' stamp: 'mt 6/3/2010 12:21'!
waitForSignal: aSignal

	| signal |
	[signal notNil and: [signal first = aSignal]]
		whileFalse: [signal := self queue next].
		
	^ signal! !

AXAnnouncement subclass: #SignalBenchmarkAnnouncement
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Benchmarks'!

!SignalBenchmarkAnnouncement class methodsFor: 'as yet unclassified' stamp: 'mt 6/10/2010 18:11:56.578'!
value: aValue

	^ self new value: aValue! !

!SignalBenchmarkAnnouncement methodsFor: 'accessing' stamp: 'mt 6/10/2010 18:07:58.242'!
value

	^ value! !

!SignalBenchmarkAnnouncement methodsFor: 'accessing' stamp: 'mt 6/10/2010 18:08:15.164'!
value: aValue

	value := aValue.! !

!OBMorphicIcons methodsFor: '*signals-ob-morphic' stamp: 'mt 6/5/2010 20:56:39.625'!
signal

	^ Form
		fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self signalContents readStream)! !

!OBMorphicIcons methodsFor: '*signals-ob-morphic' stamp: 'mt 6/9/2010 09:01:34.529'!
signal2

	^ Form
		fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self signal2Contents readStream)! !

!OBMorphicIcons methodsFor: '*signals-ob-morphic' stamp: 'mt 6/9/2010 09:01:27.741'!
signal2Contents

	^ 'iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWdVznAAAAGXRFWHRTb2Z0d2FyZQBBZG9i
ZSBJbWFnZVJlYWR5ccllPAAAALZJREFUeNpi/P//PwMpgCW94zy6WD0avxFFAxaFDTgMb0TW
UI+kEF0DMr+RCY8kVjEWJNPx2QCXZ4SFEtDzcI1M//8y/QdJMTL9h4nNrDBsRNGgs5ARZqIG
EPsB8RMg3gvEL0GCV+L/Y4QSCCgA8SIgNgHif0C8HYgTgfgNTAGyp0FsMyA2ANkMxMxA7AKy
HMrH0AAy8ToQP0ISewhzEraIA4GbQJwGxGFAzA513l0ghqcfgAADAKZnM8YGNuTNAAAAAElF
TkSuQmCC'! !

!OBMorphicIcons methodsFor: '*signals-ob-morphic' stamp: 'mt 6/5/2010 20:55:14.196'!
signalContents

	^ 'iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWdVznAAAAGXRFWHRTb2Z0d2FyZQBBZG9i
ZSBJbWFnZVJlYWR5ccllPAAAALZJREFUeNpi/P//PwMpgCW94zy6WD0avxFFAxaFDTgMb0TW
UI+kEF0DMr+RCY8kVjEWJNPx2QCXZ4SFEtDzcI1M//8y/QdJMTL9h4nNrDBsRNFw04ERZqIG
EPsB8RMg3gvEL0GC6gf+Y4QSCCgA8SIgNgHif0C8HYgTgfgNTAGyp0FsMyA2ANkMxMxA7ALE
OlA+hgaQideB+BGS2EOYk7BFHNgrQJwGxGFAzA513l0ghqcfgAADAK/nM8Z9MCmQAAAAAElF
TkSuQmCC'! !

!SystemNavigation methodsFor: '*signals-reflection' stamp: 'mt 6/6/2010 09:52:02.33'!
allConnectionsOn: aSignal

	^ self
		allConnectionsOn: aSignal
		from: Object! !

!SystemNavigation methodsFor: '*signals-reflection' stamp: 'mt 6/6/2010 10:02:48.59'!
allConnectionsOn: aSignal from: aClass
	"Return all methods of a class that contain connections on the given signal."

	| connectionSelectors result tmp |
	tmp := self allCallsOn: aSignal from: aClass. "Fetch connections AND signal sends.."
	
	result := OrderedCollection new.
	connectionSelectors := Object selectors select: [:sel | sel startsWith: #connect:].
	connectionSelectors do: [:sel | 
		result addAll: (self allCallsOn: sel from: aClass)].
	
	^ result select: [:e | tmp includes: e]. "Return the intersection"! !

!SystemNavigation methodsFor: '*signals-reflection' stamp: 'mt 6/6/2010 10:00:42.534'!
allSignalEmitsFrom: aSenderClass signal: aSignal

	| connectionSelectors result |
	result := self allCallsOn: aSignal from: aSenderClass. "Fetch connections AND signal sends.."
	
	connectionSelectors := Object selectors select: [:sel | sel startsWith: #connect:].
	connectionSelectors do: [:sel | 
		| calls |
		calls := self allCallsOn: sel from: aSenderClass.
		"Hint: Lose signal sends that are in the same message as the connection. "
		result := result reject: [:e | calls includes: e]].
	
	^ result! !

OrderedCollection subclass: #SignalMethodWrapperRepository
	instanceVariableNames: 'mutex'
	classVariableNames: 'Repository'
	poolDictionaries: ''
	category: 'Signals-Wrapper'!

!SignalMethodWrapperRepository class methodsFor: 'instance' stamp: 'mt 6/10/2010 22:41:31.851'!
critical: aBlock

	^ self instance critical: aBlock! !

!SignalMethodWrapperRepository class methodsFor: 'instance' stamp: 'mt 6/10/2010 22:18:23.411'!
instance

	^ Repository ifNil: [Repository := self new] ! !

!SignalMethodWrapperRepository class methodsFor: 'instance' stamp: 'mt 6/14/2010 21:47:55.696'!
reset

	Repository := nil.! !

!SignalMethodWrapperRepository methodsFor: 'critical' stamp: 'mt 6/10/2010 22:30:06.176'!
critical: aBlock

	mutex ifNil: [mutex := Mutex new].
	^ mutex critical: aBlock! !

MwMethodWrapper subclass: #SignalMethodWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Wrapper'!

!SignalMethodWrapper commentStamp: 'mt 6/10/2010 22:01:14.237' prior: 0!
I am able to turn any method into a signal without the need of changing the code of that method.!

!SignalMethodWrapper class methodsFor: 'convertig' stamp: 'mt 6/14/2010 21:55:02.808'!
checkAndConvertSignal: aSignalAndClass
	
	(SignalConnectionsRepository checkSignal: aSignalAndClass first inClass: aSignalAndClass second)
		ifFalse:[self convertToSelector: aSignalAndClass first inClass: aSignalAndClass second].! !

!SignalMethodWrapper class methodsFor: 'convertig' stamp: 'mt 6/14/2010 21:54:57.817'!
convertToSelector: aSignal inClass: aClass
	
	SignalMethodWrapperRepository critical: [ | wrapper |
		wrapper := (SignalMethodWrapperRepository instance
			detect: [:w | w wrappedClass = aClass and: [w clientSelector = aSignal]]
			ifNone: [^ self "Is no wrapped signal"]).
		wrapper uninstall.
		SignalMethodWrapperRepository instance remove: wrapper].! !

!SignalMethodWrapper class methodsFor: 'convertig' stamp: 'mt 6/17/2010 15:37:25.871'!
convertToSignal: aSelector inClass: aClass

	| wrapper |
	(aClass whichClassIncludesSelector: aSelector) ifNotNilDo: [ :c |
		c >> aSelector literalsDo: [ :literal |
			(literal == #emit or: [literal == #emitAlways])
				ifTrue: [^ self "Already a signal." ] ] ].
	
	wrapper := self on: aSelector inClass: aClass.
	
	SignalMethodWrapperRepository critical: [
		(SignalMethodWrapperRepository instance anySatisfy: [:w |
			w wrappedClass = aClass and: [w clientSelector = aSelector]]) ifTrue: [^ self "Already a signal via wrapping."].
		SignalMethodWrapperRepository instance add: wrapper].
	
	wrapper install! !

!SignalMethodWrapper methodsFor: 'evaluating' stamp: 'mt 6/10/2010 22:10:34.366'!
valueWithReceiver: anObject arguments: someArguments

	^ [self clientMethod
		valueWithReceiver: anObject
		arguments: someArguments.
	] ensure: [self
		emitSignal: self clientSelector
		arguments: someArguments
		sender: anObject]! !

TestCase subclass: #SignalKeyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalKeyTest methodsFor: 'tests' stamp: 'mt 6/14/2010 20:02:34.653'!
test01KeyInDictionary

	| dict |
	dict := Dictionary new.
	dict at: (SignalKey signal: #valueChanged: sender: SignalProducer new) put: #foobar.
	
	self assert: #foobar equals: (dict at: (SignalKey signal: #valueChanged: sender: SignalProducer new)).! !

TestCase subclass: #SignalReflectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalReflectionTest methodsFor: 'testing' stamp: 'mt 6/6/2010 11:57:12.056'!
test01FindConnections

	self
		assert: #(test03 test15 test17 test19) asOrderedCollection
		equals: ((SystemNavigation default
			allConnectionsOn: #newSignalProduced
			from: SignalTest) collect: [:methodRef | (methodRef methodSymbol first: 6) asSymbol]).! !

!SignalReflectionTest methodsFor: 'testing' stamp: 'mt 6/17/2010 15:16:46.095'!
test02FindSignalEmits

	self
		assert: #(value: valueInBlock: valueEx:) asOrderedCollection
		equals: ((SystemNavigation default
			allSignalEmitsFrom: SignalProducer
			signal: #valueChanged:) collect: [:methodRef | methodRef methodSymbol]).! !

TestCase subclass: #SignalTest
	instanceVariableNames: 'producer consumer process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalTest methodsFor: 'running' stamp: 'mt 6/2/2010 22:42'!
setUp

	consumer := SignalConsumer new.
	producer := SignalProducer new.! !

!SignalTest methodsFor: 'running' stamp: 'mt 6/3/2010 08:19'!
tearDown

	super tearDown.
	(process notNil and: [process isTerminated not]) ifTrue: [process terminate].! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/2/2010 22:00'!
test01ConnectDisconnect

	self assert: producer signalConnections isEmpty.
	consumer connect: producer signal: #valueChanged: toSelector: #value:.
	self assert: 1 equals: producer signalConnections size.
	
	self
		assert: (producer disconnectSignal: #valueChanged:);
		assert: producer signalConnections isEmpty;
		deny: (producer disconnectSignal: #valueChanged:).
		
	self connect: producer signal: #valueChanged: to: consumer selector: #value:.
	self assert: 1 equals: producer signalConnections size.

	self
		assert: (producer disconnectSignal: #valueChanged: from: consumer);
		assert: producer signalConnections isEmpty;
		deny: (producer disconnectSignal: #valueChanged: from: consumer).
		
	self connect: producer signal: #valueChanged: to: consumer selector: #value:.
	
	self
		assert: (producer disconnectSignal: #valueChanged: from: consumer selector: #value:);
		assert: producer signalConnections isEmpty;
		deny: (producer disconnectSignal: #valueChanged: from: consumer selector: #value:).
		
	self connect: producer signal: #valueChanged: to: consumer selector: #value:.
	self 
		deny: (producer disconnectSignal: #shouldNotExist);
		deny: producer signalConnections isEmpty.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/2/2010 22:00'!
test02ConnectError

	self
		should: [consumer connect: producer signal: #shouldNotExist toSelector: #toggleSwitch] raise: Error;
		should: [consumer connect: producer signal: #valueChanged: toSelector: #shouldNotExist] raise: Error;
		should: [self connect: producer signal: #shouldNotExist to: consumer selector: #toggleSwitch] raise: Error;
		should: [self connect: producer signal: #valueChanged: to: consumer selector: #shouldNotExist] raise: Error.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:01.776'!
test03Signaling

	consumer connect: producer signal: #valueChanged: toSelector: #value:.
	consumer connect: producer signal: #newSignalProduced toSelector: #toggleSwitch.
	
	self
		assert: consumer value isNil;
		assert: #off equals: consumer switchState.	
	producer value: 'foobar'.
	producer produceSignal.
	self
		assert: consumer value = 'foobar';
		assert: #on equals: consumer switchState.	
	
	producer disconnectAllSignals.
	producer value: 'blubb'.
	self
		assert: consumer value = 'foobar';
		assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:06.376'!
test04Patterns

	consumer connect: producer signal: #valueChanged: toSelector: #toggleSwitch pattern: #().
	
	self assert:#off equals: consumer switchState.	
	producer value: 'foobar'.
	self assert:#on equals: consumer switchState.	! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:13.531'!
test05PatternsReorder

	consumer connect: producer signal: #valueChanged:and: toSelector: #value: pattern: #(2).
	
	self assert: consumer value isNil.
	producer value: 'foobar' and: 'blubb'.
	self assert: 'blubb' equals: consumer value.	! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:22.928'!
test06DoubleConnection

	consumer connect: producer signal: #valueChanged: toSelector: #value:.
	consumer connect: producer signal: #valueChanged: toSelector: #toggleSwitch pattern: #().
	
	self
		assert: consumer value isNil;
		assert: #off equals: consumer switchState.	
	
	producer value: 'foobar'.
	
	self
		assert: consumer value = 'foobar';
		assert: #on equals: consumer switchState.	! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/4/2010 13:39'!
test07PatternError

	self
		should: [consumer connect: producer signal: #valueChanged: toSelector: #toggleSwitch pattern: #(1)] raise: Error;
		should: [consumer connect: producer signal: #valueChanged: toSelector: #value: pattern: #(2)] raise: Error;
		should: [consumer connect: producer signal: #valueChanged:with: toSelector: #value: pattern: #(2 1)] raise: Error;
		should: [consumer connect: producer signal: #valueChanged: toSelector: #value: pattern: #()] raise: Error.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:34.539'!
test08QueuedConnection

	| queue result |
	queue := SharedQueue new.
	process := [queue next value. result := consumer value] forkAt: 60.
	
	consumer connect: producer signal: #valueChanged: toSelector: #value: queue: queue.
	producer value: #QueuedConnectionsAreCool.
	
	Processor yield. "Let the result be set."
	
	self assert: #QueuedConnectionsAreCool equals: result.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:48.546'!
test09QueuedConnectionToUI

	consumer
		connect: producer
		signal: #valueChanged:
		toSelector: #value:
		queue: WorldState deferredUIMessages.
		
	producer value: #QueuedConnectionsAreCool.
	
	self assert: nil equals: consumer value.
	World doOneCycle. "Let the result be set. Do it manually here because we ARE already in the UI process."
	self assert: #QueuedConnectionsAreCool equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:54.378'!
test10BlockingConnection

	| result |
	consumer
		connect: producer
		signal: #valueChanged:
		toSelector: #value:
		queue: WorldState deferredUIMessages
		blocking: true.
		
	process := [producer value: #QueuedConnectionsAreCool. result := 'blubb'] forkAt: 60.
	Processor yield.
	
	self assert: nil equals: result.
	
	World doOneCycle. "Deliver the signal."
	Processor yield. "Let the result be set."
	
	self assert: 'blubb' equals: result.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:58.852'!
test11SignalSpy

	| spy |
	spy := SignalSpy onSender: producer signal: #valueChanged:.
	producer value: #DontSpyOnMe.	
	self assert: {#valueChanged:. {#DontSpyOnMe}} equals: spy waitForNextSignal.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:07:04.379'!
test12SignalSpyConcurrent

	| spy |
	spy := SignalSpy onSender: producer signal: #valueChanged:.
	process := [producer value: #DontSpyOnMe] fork.
	self assert: {#valueChanged:. {#DontSpyOnMe}} equals: spy waitForNextSignal.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:07:15.917'!
test13SignalSpySpecificSignal

	| spy |
	spy := SignalSpy onSender: producer signal: #valueChanged:.
	process := [
		producer produceSignal. "will be discarded"
		producer value: #DontSpyOnMe] fork.
	self assert: {#valueChanged:. {#DontSpyOnMe}} equals: (spy waitForSignal: #valueChanged:).! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:07:22.989'!
test14DefaultUIProcessing
	"Check whether default connections out of the ui process will only be processed in the ui process."
	
	self assert: Project uiProcess equals: Processor activeProcess.
	
	consumer
		connect: producer
		signal: #valueChanged:
		toSelector: #value:.
		
	process := [producer value: #QueuedConnectionsAreCool] forkAt: 60.
	Processor yield.
	
	self assert: nil equals: consumer value.
	
	World doOneCycle. "Deliver the signal."
	
	self assert: #QueuedConnectionsAreCool equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/4/2010 13:28'!
test15ArgumentCountError

	self
		should: [consumer connect: producer signal: #newSignalProduced toSelector: #value:]
		raise: Error.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:07:30.717'!
test16ArgumentAutoTruncate

	consumer connect: producer signal: #valueChanged: toSelector: #toggleSwitch.
	
	self assert: #off equals: consumer switchState.
	producer value: #foobar.
	self assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:07:35.016'!
test17ConnectSignalToSignal

	producer connect: producer signal: #valueChanged: toSelector: #newSignalProduced.
	consumer connect: producer signal: #newSignalProduced toSelector: #toggleSwitch.
	
	self assert: #off equals: consumer switchState.
	producer value: #foobar.
	self assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:34:17.452'!
test18SignalFromOutsideError
	"There should be an error if another class tries to emit a signal from the outside."
	
	self
		should: [producer valueChanged: #foobar] raise: Error;
		should: [[producer valueChanged: #foobar] value] raise: Error;
		shouldnt: [producer valueInBlock: #foobar] raise: Error.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 11:46:34.493'!
test19UseSenderAsArgument
	
	consumer connect: producer signal: #newSignalProduced toSelector: #value: pattern: #(0).
	producer produceSignal.
	
	self assert: producer equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 12:19:57.244'!
test20BadSignal
	
	consumer connect: producer signal: #newSignalProducedInBlock toSelector: #toggleSwitch.
	producer produceBadSignal.
	
	self assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 22:34:30.515'!
test21AutoConnections
	
	| c |
	c := SignalConsumerAuto new.
	
	self
		assert: #off equals: c switchState;
		assert: nil equals: c value.
		
	c producer produceSignal; value: #foobar.
	
	self
		assert: #on equals: c switchState;
		assert: #foobar equals: c value.
		
	c producer value: #foobar and: #blubb.
	
	self
		assert: #on equals: c switchState;
		assert: #blubb equals: c value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 23:07:24.806'!
test22AutoConnectionsTwo
	
	| c |
	c := SignalConsumerAuto new.
	
	self assert: #off equals: c switchState.
		
	c producerTwo produceSignal.
	
	self assert: #on equals: c switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/9/2010 08:57:42.069'!
test23EmitPublicSignal

	consumer connect: producer signal: #valueChanged2: toSelector: #value:.
	
	self assert: nil equals: consumer value.
	producer valueChanged2: #foobar.
	self assert: #foobar equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/13/2010 10:31:07.741'!
test24ConnectSignalToSignal2

	consumer connect: producer signal: #valueChanged: toSelector: #valueConsumed.
	consumer connect: consumer signal: #valueConsumed toSelector: #toggleSwitch.
	
	self assert: #off equals: consumer switchState.
	producer value: #foobar.
	self assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/14/2010 21:19:42.768'!
test25WeakRepositoryFinalization

	SignalConnectionsRepository reset.

	SignalConsumer new
		connect: SignalProducer new
		signal: #valueChanged:
		toSelector: #valueConsumed.
	
	"Let's finalize."
	Smalltalk garbageCollect.
	Processor yield.

	self assert: SignalConnectionsRepository instance isEmpty.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/17/2010 15:14:25.87'!
test26BaseClassSignals

	| producerEx |
	producerEx := SignalProducerEx new.
	
	consumer connect: producerEx signal: #valueChanged: toSelector: #value:.
	
	self assert: nil equals: consumer value.
	producerEx value: #foobar.
	self assert: #foobar equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/17/2010 15:40:50.535'!
test27BaseClassSignalsEx

	| producerEx |
	producerEx := SignalProducerEx new.
	
	consumer connect: producerEx signal: #valueChanged: toSelector: #value:.
	
	self assert: nil equals: consumer value.
	producerEx valueEx: #foobar.
	self assert: #foobar equals: consumer value.! !

TestCase subclass: #SignalWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Wrapper-Tests'!

!SignalWrapperTest methodsFor: 'running' stamp: 'mt 6/14/2010 21:43:29.742'!
setUp

	super setUp.
	Smalltalk garbageCollect.
	Processor yield.! !

!SignalWrapperTest methodsFor: 'running' stamp: 'mt 6/14/2010 21:42:54.346'!
tearDown

	super tearDown.
	Smalltalk garbageCollect.
	Processor yield.! !

!SignalWrapperTest methodsFor: 'tests' stamp: 'mt 6/14/2010 22:36:51.161'!
test01ConvertToSignal

	| producer consumer |
	producer := SignalProducerByWrapping new.
	consumer := SignalConsumer new.

	self
		assert: 0
		equals: SignalProducerByWrapping wrapperCount.
		
	consumer connect: producer signal: #value: toSelector: #value:.
	
	self
		assert: 1
		equals: SignalProducerByWrapping wrapperCount.
		
	consumer connect: producer signal: #value: toSelector: #value:.
	
	self
		assert: 1
		equals: SignalProducerByWrapping wrapperCount.! !

!SignalWrapperTest methodsFor: 'tests' stamp: 'mt 6/14/2010 21:56:08.093'!
test02ConnectToConvertedSignal

	| producer consumer |
	producer := SignalProducerByWrapping new.
	consumer := SignalConsumer new.
		
	consumer connect: producer signal: #value: toSelector: #value:.
	producer value: #foobar.
	
	self
		assert: #foobar
		equals: consumer value.! !

!SignalWrapperTest methodsFor: 'tests' stamp: 'mt 6/14/2010 22:08:02.979'!
test03ConnectConvertedSignalToConvertedSignal

	| producer consumer |
	producer := SignalProducerByWrapping new.
	consumer := SignalConsumer new.
	
	"Transform #value: from consumer to signal."
	consumer connect: consumer signal: #value: toSelector: #toggleSwitch.
	"Transform #value: from producer to signal."
	consumer connect: producer signal: #value: toSelector: #value:.
	producer value: #foobar.
	
	self
		assert: #on
		equals: consumer switchState.! !
